@@ -133,7 +133,7 @@ V3_AUTH_RESPONSE = json.dumps({
 CINDER_EXAMPLES = {
     "versions": [
         {
-            "status": "CURRENT",
+            "status": discover.Status.CURRENT,
             "updated": "2012-01-04T11:33:21Z",
             "id": "v1.0",
             "links": [
@@ -144,7 +144,7 @@ CINDER_EXAMPLES = {
             ]
         },
         {
-            "status": "CURRENT",
+            "status": discover.Status.CURRENT,
             "updated": "2012-11-21T11:33:21Z",
             "id": "v2.0",
             "links": [
@@ -155,7 +155,7 @@ CINDER_EXAMPLES = {
             ]
         },
         {
-            "status": "CURRENT",
+            "status": discover.Status.CURRENT,
             "updated": "2012-11-21T11:33:21Z",
             "id": "v3.0",
             "version": "3.27",
@@ -179,7 +179,7 @@ CINDER_EXAMPLES = {
 GLANCE_EXAMPLES = {
     "versions": [
         {
-            "status": "CURRENT",
+            "status": discover.Status.CURRENT,
             "id": "v2.2",
             "links": [
                 {
@@ -189,7 +189,7 @@ GLANCE_EXAMPLES = {
             ]
         },
         {
-            "status": "SUPPORTED",
+            "status": discover.Status.SUPPORTED,
             "id": "v2.1",
             "links": [
                 {
@@ -199,7 +199,7 @@ GLANCE_EXAMPLES = {
             ]
         },
         {
-            "status": "SUPPORTED",
+            "status": discover.Status.SUPPORTED,
             "id": "v2.0",
             "links": [
                 {
@@ -209,7 +209,7 @@ GLANCE_EXAMPLES = {
             ]
         },
         {
-            "status": "CURRENT",
+            "status": discover.Status.CURRENT,
             "id": "v1.1",
             "links": [
                 {
@@ -219,7 +219,7 @@ GLANCE_EXAMPLES = {
             ]
         },
         {
-            "status": "SUPPORTED",
+            "status": discover.Status.SUPPORTED,
             "id": "v1.0",
             "links": [
                 {
@@ -323,52 +323,63 @@ class DiscoverUtils(utils.TestCase):
 
     def test_version_args(self):
         """Validate _normalize_version_args."""
-        def assert_min_max(in_ver, in_min, in_max, out_min, out_max):
+        def assert_min_max(in_ver, in_min, in_max, in_type, out_min, out_max):
             self.assertEqual(
                 (out_min, out_max),
-                discover._normalize_version_args(in_ver, in_min, in_max))
+                discover._normalize_version_args(
+                    in_ver, in_min, in_max, service_type=in_type))
 
-        def normalize_raises(ver, min, max):
-            self.assertRaises(ValueError,
-                              discover._normalize_version_args, ver, min, max)
+        def normalize_raises(ver, min, max, in_type):
+            self.assertRaises(
+                ValueError,
+                discover._normalize_version_args,
+                ver, min, max, service_type=in_type)
 
-        assert_min_max(None, None, None,
+        assert_min_max(None, None, None, None,
                        None, None)
-        assert_min_max(None, None, 'v1.2',
+        assert_min_max(None, None, 'v1.2', None,
                        None, (1, 2))
-        assert_min_max(None, 'v1.2', 'latest',
+        assert_min_max(None, 'v1.2', 'latest', None,
                        (1, 2), (discover.LATEST, discover.LATEST))
-        assert_min_max(None, 'v1.2', '1.6',
+        assert_min_max(None, 'v1.2', '1.6', None,
                        (1, 2), (1, 6))
-        assert_min_max(None, 'v1.2', '1.latest',
+        assert_min_max(None, 'v1.2', '1.latest', None,
                        (1, 2), (1, discover.LATEST))
-        assert_min_max(None, 'latest', 'latest',
+        assert_min_max(None, 'latest', 'latest', None,
                        (discover.LATEST, discover.LATEST),
                        (discover.LATEST, discover.LATEST))
-        assert_min_max(None, 'latest', None,
+        assert_min_max(None, 'latest', None, None,
                        (discover.LATEST, discover.LATEST),
                        (discover.LATEST, discover.LATEST))
-        assert_min_max(None, (1, 2), None,
+        assert_min_max(None, (1, 2), None, None,
                        (1, 2), (discover.LATEST, discover.LATEST))
-        assert_min_max('', ('1', '2'), (1, 6),
+        assert_min_max('', ('1', '2'), (1, 6), None,
                        (1, 2), (1, 6))
-        assert_min_max(None, ('1', '2'), (1, discover.LATEST),
+        assert_min_max(None, ('1', '2'), (1, discover.LATEST), None,
                        (1, 2), (1, discover.LATEST))
-        assert_min_max('v1.2', '', None,
+        assert_min_max('v1.2', '', None, None,
                        (1, 2), (1, discover.LATEST))
-        assert_min_max('1.latest', None, '',
+        assert_min_max('1.latest', None, '', None,
                        (1, discover.LATEST), (1, discover.LATEST))
-        assert_min_max('v1', None, None,
+        assert_min_max('v1', None, None, None,
                        (1, 0), (1, discover.LATEST))
-        assert_min_max('latest', None, None,
+        assert_min_max('latest', None, None, None,
                        (discover.LATEST, discover.LATEST),
                        (discover.LATEST, discover.LATEST))
+        assert_min_max(None, None, 'latest', 'volumev2',
+                       (2, 0), (2, discover.LATEST))
+        assert_min_max(None, None, None, 'volumev2',
+                       (2, 0), (2, discover.LATEST))
 
-        normalize_raises('v1', 'v2', None)
-        normalize_raises('v1', None, 'v2')
-        normalize_raises(None, 'latest', 'v1')
-        normalize_raises(None, 'v1.2', 'v1.1')
-        normalize_raises(None, 'v1.2', 1)
+        normalize_raises('v1', 'v2', None, None)
+        normalize_raises('v1', None, 'v2', None)
+        normalize_raises(None, 'latest', 'v1', None)
+        normalize_raises(None, 'v1.2', 'v1.1', None)
+        normalize_raises(None, 'v1.2', 1, None)
+        normalize_raises('v2', None, None, 'volumev3')
+        normalize_raises('v3', None, None, 'volumev2')
+        normalize_raises(None, 'v2', None, 'volumev3')
+        normalize_raises(None, None, 'v3', 'volumev2')
 
     def test_version_to_string(self):
         def assert_string(out, inp):
@@ -384,14 +395,14 @@ class DiscoverUtils(utils.TestCase):
 
     def test_version_between(self):
         def good(minver, maxver, cand):
-            self.assertTrue(discover._version_between(minver, maxver, cand))
+            self.assertTrue(discover.version_between(minver, maxver, cand))
 
         def bad(minver, maxver, cand):
-            self.assertFalse(discover._version_between(minver, maxver, cand))
+            self.assertFalse(discover.version_between(minver, maxver, cand))
 
         def exc(excls, minver, maxver, cand):
             self.assertRaises(excls,
-                              discover._version_between, minver, maxver, cand)
+                              discover.version_between, minver, maxver, cand)
 
         # candidate required
         exc(ValueError, (1, 0), (1, 0), None)
@@ -480,6 +491,18 @@ class VersionDataTests(utils.TestCase):
 
             self.assertTrue(mock.called_once)
 
+    def test_version_data_unknown(self):
+        discovery_fixture = fixture.V3Discovery(V3_URL)
+        discovery_fixture.status = 'hungry'
+        discovery_doc = _create_single_version(discovery_fixture)
+
+        self.requests_mock.get(V3_URL, status_code=200, json=discovery_doc)
+
+        disc = discover.Discover(self.session, V3_URL)
+        clean_data = disc.version_data(allow_unknown=True)
+
+        self.assertEqual(discover.Status.UNKNOWN, clean_data[0]['status'])
+
     def test_version_data_individual(self):
         mock = self.requests_mock.get(V3_URL,
                                       status_code=200,
@@ -497,6 +520,7 @@ class VersionDataTests(utils.TestCase):
 
         for v in clean_data:
             self.assertEqual(v['version'], (3, 0))
+            self.assertEqual(v['status'], discover.Status.CURRENT)
             self.assertEqual(v['raw_status'], 'stable')
             self.assertEqual(v['url'], V3_URL)
 
@@ -510,7 +534,7 @@ class VersionDataTests(utils.TestCase):
                 "versions": [
                     dict(
                         {
-                            "status": "CURRENT",
+                            "status": discover.Status.CURRENT,
                             "id": "v2.2",
                             "links": [
                                 {
@@ -536,7 +560,8 @@ class VersionDataTests(utils.TestCase):
                             'collection': None,
                             'version': (2, 2),
                             'url': V3_URL,
-                            'raw_status': 'CURRENT',
+                            'status': discover.Status.CURRENT,
+                            'raw_status': discover.Status.CURRENT,
                         },
                         **versions_out
                     )
@@ -785,7 +810,7 @@ class VersionDataTests(utils.TestCase):
         self.assertEqual(3, len(raw_data))
 
         for v in raw_data:
-            self.assertEqual(v['status'], 'CURRENT')
+            self.assertEqual(v['status'], discover.Status.CURRENT)
             if v['id'] == 'v1.0':
                 self.assertEqual(v['updated'], '2012-01-04T11:33:21Z')
             elif v['id'] == 'v2.0':
@@ -808,7 +833,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (1, 0),
                 'url': v1_url,
-                'raw_status': 'CURRENT',
+                'status': discover.Status.CURRENT,
+                'raw_status': discover.Status.CURRENT,
             },
             {
                 'collection': None,
@@ -818,7 +844,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (2, 0),
                 'url': v2_url,
-                'raw_status': 'CURRENT',
+                'status': discover.Status.CURRENT,
+                'raw_status': discover.Status.CURRENT,
             },
             {
                 'collection': BASE_URL,
@@ -828,7 +855,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': u'2019-12-31',
                 'version': (3, 0),
                 'url': v3_url,
-                'raw_status': 'CURRENT',
+                'status': discover.Status.CURRENT,
+                'raw_status': discover.Status.CURRENT,
             },
         ])
 
@@ -836,7 +864,7 @@ class VersionDataTests(utils.TestCase):
                         disc.versioned_data_for(min_version='v2.0',
                                                 max_version='v2.latest')):
             self.assertEqual((2, 0), version['version'])
-            self.assertEqual('CURRENT', version['raw_status'])
+            self.assertEqual(discover.Status.CURRENT, version['raw_status'])
             self.assertEqual(v2_url, version['url'])
 
         for version in (disc.data_for(1),
@@ -844,7 +872,7 @@ class VersionDataTests(utils.TestCase):
                             min_version=(1,),
                             max_version=(1, discover.LATEST))):
             self.assertEqual((1, 0), version['version'])
-            self.assertEqual('CURRENT', version['raw_status'])
+            self.assertEqual(discover.Status.CURRENT, version['raw_status'])
             self.assertEqual(v1_url, version['url'])
 
         self.assertIsNone(disc.url_for('v4'))
@@ -875,9 +903,9 @@ class VersionDataTests(utils.TestCase):
 
         for v in raw_data:
             if v['id'] in ('v2.2', 'v1.1'):
-                self.assertEqual(v['status'], 'CURRENT')
+                self.assertEqual(v['status'], discover.Status.CURRENT)
             elif v['id'] in ('v2.1', 'v2.0', 'v1.0'):
-                self.assertEqual(v['status'], 'SUPPORTED')
+                self.assertEqual(v['status'], discover.Status.SUPPORTED)
             else:
                 self.fail("Invalid version found")
 
@@ -893,7 +921,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (1, 0),
                 'url': v1_url,
-                'raw_status': 'SUPPORTED',
+                'status': discover.Status.SUPPORTED,
+                'raw_status': discover.Status.SUPPORTED,
             },
             {
                 'collection': None,
@@ -903,7 +932,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (1, 1),
                 'url': v1_url,
-                'raw_status': 'CURRENT',
+                'status': discover.Status.CURRENT,
+                'raw_status': discover.Status.CURRENT,
             },
             {
                 'collection': None,
@@ -913,7 +943,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (2, 0),
                 'url': v2_url,
-                'raw_status': 'SUPPORTED',
+                'status': discover.Status.SUPPORTED,
+                'raw_status': discover.Status.SUPPORTED,
             },
             {
                 'collection': None,
@@ -923,7 +954,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (2, 1),
                 'url': v2_url,
-                'raw_status': 'SUPPORTED',
+                'status': discover.Status.SUPPORTED,
+                'raw_status': discover.Status.SUPPORTED,
             },
             {
                 'collection': None,
@@ -933,7 +965,8 @@ class VersionDataTests(utils.TestCase):
                 'not_before': None,
                 'version': (2, 2),
                 'url': v2_url,
-                'raw_status': 'CURRENT',
+                'status': discover.Status.CURRENT,
+                'raw_status': discover.Status.CURRENT,
             },
         ])
 
@@ -943,7 +976,9 @@ class VersionDataTests(utils.TestCase):
                                 min_version=ver,
                                 max_version=(2, discover.LATEST))):
                 self.assertEqual((2, 2), version['version'])
-                self.assertEqual('CURRENT', version['raw_status'])
+                self.assertEqual(
+                    discover.Status.CURRENT, version['raw_status']
+                )
                 self.assertEqual(v2_url, version['url'])
                 self.assertEqual(v2_url, disc.url_for(ver))
 
@@ -953,7 +988,9 @@ class VersionDataTests(utils.TestCase):
                                 min_version=ver,
                                 max_version=(1, discover.LATEST))):
                 self.assertEqual((1, 1), version['version'])
-                self.assertEqual('CURRENT', version['raw_status'])
+                self.assertEqual(
+                    discover.Status.CURRENT, version['raw_status']
+                )
                 self.assertEqual(v1_url, version['url'])
                 self.assertEqual(v1_url, disc.url_for(ver))
 
